<?php

namespace Drupal\email_manager;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Component\Serialization\Yaml;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\contact\Entity\ContactForm;
use Drupal\contact\Entity\Message;
use Drupal\user\Entity\User;

/**
 * Service to collect email keys and their associated modules.
 */
class EmailManagerKeyCollector {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs an EmailManagerKeyCollector object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   */
  public function __construct(ModuleHandlerInterface $module_handler) {
    $this->moduleHandler = $module_handler;
  }

  /**
   * Collects all email keys and their associated modules.
   *
   * @return array
   *   An array of email keys in the format "module:key" => "Label (Module)".
   */
  public function getEmailKeys() {
    $cache = \Drupal::cache()->get('email_manager:email_keys');
    if ($cache) {
      return $cache->data;
    }

    $email_keys = [];

    // Step 1: Collect keys from manually defined EmailKey entities.
    $email_key_entities = \Drupal::entityTypeManager()->getStorage('email_key')->loadMultiple();
    foreach ($email_key_entities as $email_key) {
      $module = $email_key->get('module');
      $key = $email_key->get('key');
      $label = $email_key->label();
      $email_keys["$module:$key"] = "$label ($module)";
    }

    // Step 2: Collect keys from configuration files.
    $modules = array_keys($this->moduleHandler->getModuleList());
    foreach ($modules as $module) {
      $config_file = $this->moduleHandler->getModule($module)->getPath() . "/$module.email_manager.yml";
      if (file_exists($config_file)) {
        $config = Yaml::decode(file_get_contents($config_file));
        if (!empty($config['email_keys']) && is_array($config['email_keys'])) {
          foreach ($config['email_keys'] as $key => $label) {
            if (!isset($email_keys["$module:$key"])) {
              $email_keys["$module:$key"] = "$label ($module)";
            }
          }
        }
      }
    }

    // Step 3: Collect keys by invoking hook_mail() with module-specific safe keys.
    $module_safe_keys = [
      'user' => [
        'password_reset' => 'Password Reset',
        'register_admin_created' => 'Admin Created Account',
        'register_no_approval_required' => 'No Approval Required',
        'status_activated' => 'Account Activated',
      ],
      // Removed 'contact' module support due to persistent errors.
    ];

    // Load a real user entity (admin user with ID 1) as the recipient.
    $recipient = User::load(1);
    if (!$recipient) {
      // Fallback: Create a temporary user entity if admin user doesn't exist.
      $recipient = User::create([
        'name' => 'email_manager_discovery_user',
        'mail' => 'discovery@example.com',
        'status' => 1,
      ]);
    }

    foreach ($modules as $module) {
      $hook_function = $module . '_mail';
      if (function_exists($hook_function)) {
        $safe_keys = $module_safe_keys[$module] ?? [];
        if (empty($safe_keys)) {
          // Skip modules without known safe keys to avoid errors.
          continue;
        }

        foreach ($safe_keys as $safe_key => $label) {
          // Initialize a minimal message to avoid errors.
          $message = [
            'subject' => new TranslatableMarkup(''),
            'body' => [],
            'headers' => [],
            'params' => [],
          ];
          $params = [
            'context' => 'email_manager_key_discovery',
            'langcode' => \Drupal::languageManager()->getCurrentLanguage()->getId(),
          ];

          // Add dummy params expected by specific modules.
          if ($module === 'user') {
            $params['account'] = $recipient;
            $params['context'] = ['url' => '', 'url_brief' => ''];
          }
          $message['params'] = $params;

          try {
            // Invoke hook_mail() with the safe key.
            $this->moduleHandler->invoke($module, 'mail', [$safe_key, &$message, $message['params']]);
            // Ensure $message['subject'] is safe after invocation.
            if (!isset($message['subject']) || $message['subject'] === null || !($message['subject'] instanceof TranslatableMarkup)) {
              $message['subject'] = new TranslatableMarkup('');
            }
            // If the subject or body was modified, the key is likely valid.
            $subject = $message['subject'] instanceof TranslatableMarkup ? $message['subject']->render() : (string) $message['subject'];
            $body = !empty($message['body']) ? implode("\n", (array) $message['body']) : '';
            if ($subject !== '' || $body !== '') {
              if (!isset($email_keys["$module:$safe_key"])) {
                $email_keys["$module:$safe_key"] = "$label ($module)";
              }
            }
          }
          catch (\Exception $e) {
            \Drupal::logger('email_manager')->error('Failed to invoke hook_mail for module @module with key @key: @error', [
              '@module' => $module,
              '@key' => $safe_key,
              '@error' => $e->getMessage(),
            ]);
          }
        }
      }
    }

    // Step 4: Fallback to common keys if none are found.
    if (empty($email_keys)) {
      $default_keys = [
        'user' => [
          'password_reset' => 'Password Reset',
          'register_admin_created' => 'Admin Created Account',
          'register_no_approval_required' => 'No Approval Required',
          'status_activated' => 'Account Activated',
        ],
        'contact' => [
          'contact_message' => 'Contact Message',
          'page_copy' => 'Page Copy',
          'user_copy' => 'User Copy',
        ],
      ];

      foreach ($default_keys as $module => $keys) {
        foreach ($keys as $key => $label) {
          $email_keys["$module:$key"] = "$label ($module)";
        }
      }
    }

    // Step 5: Allow modules to alter the list.
    $this->moduleHandler->alter('email_manager_keys', $email_keys);

    // Step 6: Sort keys alphabetically.
    asort($email_keys);

    // Step 7: Cache the results for 24 hours.
    \Drupal::cache()->set('email_manager:email_keys', $email_keys, \Drupal::time()->getRequestTime() + 86400);

    return $email_keys;
  }
}
