<?php

namespace Drupal\email_manager\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form for creating and editing email templates.
 */
class EmailTemplateForm extends EntityForm {

  /**
   * Builds the form for creating or editing an email template.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    // The entity is set by the controller (for add) or route (for edit).
    $template = $this->entity;

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#default_value' => $template->label(),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $template->id(),
      '#machine_name' => [
        'exists' => '\Drupal\email_manager\Entity\EmailTemplate::load',
      ],
      '#disabled' => !$template->isNew(),
    ];

    // Use the key collector service to get email keys.
    $email_keys = \Drupal::service('email_manager.key_collector')->getEmailKeys();
    $form['email_key'] = [
      '#type' => 'select',
      '#title' => $this->t('Email Key'),
      '#options' => $email_keys,
      '#default_value' => $template->get('module') . ':' . $template->get('key'),
      '#required' => TRUE,
      '#description' => $this->t('Select the email key to override.'),
    ];

    $form['subject'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Subject'),
      '#default_value' => $template->get('subject'),
      '#required' => TRUE,
    ];

    $form['body'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Body'),
      '#default_value' => $template->get('body'),
      '#format' => 'full_html',
      '#required' => TRUE,
      '#description' => $this->t('The email body. HTML is supported if enabled below.'),
    ];

    $form['is_html'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Send as HTML Email'),
      '#default_value' => $template->get('is_html'),
      '#description' => $this->t('If checked, the email will be sent as HTML.'),
    ];

    $form['headers'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Headers'),
      '#default_value' => $template->get('headers') ? implode("\n", $template->get('headers')) : '',
      '#description' => $this->t('Additional email headers (one per line, in the format "Header: Value"). Examples of traditional headers: "Reply-To: support@example.com", "CC: copy@example.com", "BCC: hidden@example.com". Custom headers: "X-Priority: 1", "X-Custom-Header: CustomValue", "X-Mailer: Drupal Email Manager".'),
    ];

    // Add token help if token module is enabled.
    if (\Drupal::moduleHandler()->moduleExists('token')) {
      $form['token_help'] = [
        '#type' => 'details',
        '#title' => $this->t('Available Tokens'),
        '#open' => FALSE,
      ];
      $form['token_help']['tokens'] = [
        '#theme' => 'token_tree_link',
        '#token_types' => ['user', 'site', 'email_manager'],
        '#global_types' => TRUE,
      ];
    }

    return $form;
  }

  /**
   * Saves the email template entity.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function save(array $form, FormStateInterface $form_state) {
    $template = $this->entity;
    $headers = array_filter(array_map('trim', explode("\n", $form_state->getValue('headers'))));
    $template->set('headers', $headers);

    // Split the email_key into module and key.
    [$module, $key] = explode(':', $form_state->getValue('email_key'), 2);
    $template->set('module', $module);
    $template->set('key', $key);

    // Save the body value and format.
    $body = $form_state->getValue('body');
    $template->set('body', $body['value']);
    $template->set('is_html', $form_state->getValue('is_html'));

    $status = $template->save();

    $message = $status == SAVED_NEW
      ? $this->t('Created email template %label.', ['%label' => $template->label()])
      : $this->t('Updated email template %label.', ['%label' => $template->label()]);
    \Drupal::messenger()->addStatus($message);

    $form_state->setRedirectUrl($template->toUrl('collection'));
  }
}
