<?php

namespace Drupal\email_manager\Plugin\Mail;

use Drupal\Component\Utility\Html;
use Drupal\Core\Mail\MailFormatHelper;
use Drupal\Core\Mail\MailInterface;

/**
 * Defines the Email Manager mail plugin.
 *
 * @Mail(
 *   id = "email_manager_mail",
 *   label = @Translation("Email Manager mailer"),
 *   description = @Translation("Sends emails with Email Manager templates.")
 * )
 */
class EmailManagerMail implements MailInterface {

  /**
   * Sends an email message.
   *
   * @param array $message
   *   An array containing email message data:
   *   - module: The module name.
   *   - key: The email key.
   *   - to: The recipient email address.
   *   - from: The sender email address.
   *   - subject: The email subject.
   *   - body: The email body.
   *   - headers: An array of email headers.
   *   - params: Additional parameters for token replacement.
   *
   * @return bool
   *   TRUE if the email was sent successfully, FALSE otherwise.
   */
  public function mail(array $message) {
    // Find a matching template.
    $templates = \Drupal::entityTypeManager()->getStorage('email_template')->loadMultiple();
    $template = NULL;
    foreach ($templates as $candidate) {
      if ($candidate->get('module') == $message['module'] && $candidate->get('key') == $message['key']) {
        $template = $candidate;
        break;
      }
    }

    if ($template) {
      // Replace tokens if the token module is enabled.
      $token_service = \Drupal::hasService('token') ? \Drupal::service('token') : NULL;
      $subject = $template->get('subject');
      $body = $template->get('body');

      if ($token_service) {
        $subject = $token_service->replace($subject, $message['params'], ['clear' => TRUE]);
        $body = $token_service->replace($body, $message['params'], ['clear' => TRUE]);
      }

      $message['subject'] = $subject;
      $message['body'] = [$body];
      $message['headers'] = array_merge($message['headers'], $template->get('headers'));

      // Handle HTML email if enabled.
      if ($template->get('is_html')) {
        $message['headers']['Content-Type'] = 'text/html; charset=UTF-8';
        $message['body'][0] = Html::normalize($message['body'][0]);
      }
      else {
        // Convert HTML to plain text if not HTML.
        $message['body'][0] = MailFormatHelper::htmlToText($message['body'][0]);
      }
    }

    // Use the default mail system to send the email.
    $system = \Drupal::service('plugin.manager.mail')->getInstance(['module' => 'mailsystem', 'key' => 'default']);
    return $system->mail($message);
  }
}
